package com.company.test.screen.testentity

import com.company.test.entity.TestEntity
import io.jmix.core.DataManager
import io.jmix.ui.UiComponents
import io.jmix.ui.component.ClasspathResource
import io.jmix.ui.component.Component
import io.jmix.ui.component.Image
import io.jmix.ui.model.CollectionContainer
import io.jmix.ui.screen.*
import org.springframework.beans.factory.annotation.Autowired

@UiController("jt_TestEntity.browse")
@UiDescriptor("test-entity-browse.xml")
@LookupComponent("testEntitiesTable")
class TestEntityBrowse : StandardLookup<TestEntity>() {

    @Autowired
    private lateinit var dataManager: DataManager

    @Autowired
    private lateinit var testEntitiesDc: CollectionContainer<TestEntity>

    @Autowired
    private lateinit var uiComponents: UiComponents

    @Subscribe
    private fun onAfterShow(event: AfterShowEvent) {
        val parent1 = dataManager.create(TestEntity::class.java)
        val parent2 = dataManager.create(TestEntity::class.java)
        val parent3 = dataManager.create(TestEntity::class.java)

        val testEntities1 = createSubEntities(parent1)
        val testEntities2 = createSubEntities(parent2)
        val testEntities3 = createSubEntities(parent3)
        testEntitiesDc.setItems(listOf(parent1, parent2, parent3) + testEntities1 + testEntities2 + testEntities3)
    }

    private fun createSubEntities(parent1: TestEntity?): List<TestEntity> {
        val testEntities = (1..10).map {
            val testEntity = dataManager.create(TestEntity::class.java)
            testEntity.name = "test-$it"
            testEntity.parent = parent1
            testEntity
        }
        return testEntities
    }

    @Install(to = "testEntitiesTable.previewImage", subject = "columnGenerator")
    private fun testEntitiesTablePreviewImageColumnGenerator(testEntity: TestEntity): Component? {
        if (testEntity.parent == null) return null

        val image = uiComponents.create(Image::class.java)
        image.setSource(ClasspathResource::class.java).setPath("com/company/test/image/img.png")
        image.scaleMode = Image.ScaleMode.CONTAIN
        image.setWidth("45px")
        image.setHeight("45px")
        return image
    }


}